@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -19,29 +19,36 @@
  */
 
 #include <glib.h>
+
+#ifndef __USE_POSIX
+#define _IDLE_DNS_RESOLVER_H_UNDEF_USE_POSIX
 #define __USE_POSIX
+#endif
+
 #include <sys/types.h>
 #include <netdb.h>
+
+#ifdef _IDLE_DNS_RESOLVER_H_UNDEF_USE_POSIX
 #undef __USE_POSIX
+#endif
 
 G_BEGIN_DECLS
 
 typedef struct _IdleDNSResolver IdleDNSResolver;
 typedef struct _IdleDNSResult IdleDNSResult;
 
-struct _IdleDNSResult
-{
+struct _IdleDNSResult {
 	/* as passed to socket() */
 	int ai_family;
 	int ai_socktype;
 	int ai_protocol;
-	
+
 	/* as passed to connect() */
 	struct sockaddr *ai_addr;
 	socklen_t ai_addrlen;
 
 	/* pointer to the next list member or NULL if this is the last one */
-	struct _IdleDNSResult *ai_next;
+	IdleDNSResult *ai_next;
 };
 
 void idle_dns_result_destroy(IdleDNSResult *result);
