@@ -18,6 +18,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/* For strnlen(), which is a GNU extension. */
+#define  _GNU_SOURCE
+
 #include "idle-parser.h"
 
 #include "idle-connection.h"
@@ -26,9 +29,7 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#define __USE_GNU
 #include <string.h>
-#undef __USE_GNU
 #include <stdio.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_PARSER
@@ -241,7 +242,8 @@ void idle_parser_receive(IdleParser *parser, const gchar *msg) {
 }
 
 void idle_parser_add_handler(IdleParser *parser, IdleParserMessageCode code, IdleParserMessageHandler handler, gpointer user_data) {
-	return idle_parser_add_handler_with_priority(parser, code, handler, user_data, IDLE_PARSER_HANDLER_PRIORITY_DEFAULT);
+	idle_parser_add_handler_with_priority(parser, code, handler, user_data, IDLE_PARSER_HANDLER_PRIORITY_DEFAULT);
+	return;
 }
 
 static gint _message_handler_closure_priority_compare(gconstpointer a, gconstpointer b) {
